/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvSignalEnvelope.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvControlBinop.h"
#include "HvSignalVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_1HjqA7QM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_yHWxsut0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PYBt5qSQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Mn3r4rWg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Wvey6YOC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_86d2a1cB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ax3Ut2VM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_rKxsrzXO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_twXYYft1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_beTezRCV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_y3eNYADB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_jskdWlBU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RViPIg7T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_vBdFAMJ0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_yOtYvAFk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_kSZ2vtSX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_D4YTjbPf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_wZqWgcKC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_kVcbkglQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vh14KXq2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4xJVRhbB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_237rqgix_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eS6BD6fB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YoMkxbrb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_icO7CGKi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RGkDCGKn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_y3HPROHG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_gep3HSCH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_CVFMZimB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_4msAleSW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QNUlUDVE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_e6q4g2fd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gE5a0W3J_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_uJRooLQz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_OxQR8Y3s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_nXc9xnpK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qjRUiKdi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_r7XcUro2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SrPrDKLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wwXXe58b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Sajpxm2j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0ATBIPwr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_i69hPr3i_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_T82JJoaq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vrc9B5Pt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ibpGmkT6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_rLebIf0X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_S9kK1e2P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_NvfhmRCv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_F4mBUkgS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_rpErMMLQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IuerOrUz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_91FX5Lks_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sR5pdsV7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_huvnpdf2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_nVGHXIXl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XUrCs2p9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_c2wiJ2rR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_rN0fyVfj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_EjBDtRYf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_CJROtJB5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_rGOcRt3F_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_RT0babDU;
  SignalEnvelope sEnv_wZqWgcKC;
  SignalEnvelope sEnv_kVcbkglQ;
  SignalEnvelope sEnv_OxQR8Y3s;
  SignalEnvelope sEnv_nXc9xnpK;
  SignalLine sLine_9gm0LnNw;
  SignalLine sLine_CcSQOjkO;
  ControlBinop cBinop_1HjqA7QM;
  ControlBinop cBinop_yHWxsut0;
  ControlBinop cBinop_PYBt5qSQ;
  ControlBinop cBinop_Mn3r4rWg;
  ControlBinop cBinop_ax3Ut2VM;
  ControlBinop cBinop_twXYYft1;
  ControlBinop cBinop_beTezRCV;
  ControlBinop cBinop_y3eNYADB;
  ControlVar cVar_jskdWlBU;
  ControlVar cVar_vBdFAMJ0;
  SignalVarf sVarf_CamxJcDR;
  ControlVar cVar_yOtYvAFk;
  ControlIf cIf_kSZ2vtSX;
  ControlIf cIf_D4YTjbPf;
  ControlBinop cBinop_vh14KXq2;
  ControlBinop cBinop_237rqgix;
  ControlBinop cBinop_eS6BD6fB;
  ControlBinop cBinop_RGkDCGKn;
  ControlBinop cBinop_gep3HSCH;
  SignalVarf sVarf_7WqR8stV;
  ControlVar cVar_e6q4g2fd;
  ControlIf cIf_gE5a0W3J;
  ControlIf cIf_uJRooLQz;
  ControlBinop cBinop_qjRUiKdi;
  ControlBinop cBinop_SrPrDKLI;
  ControlBinop cBinop_wwXXe58b;
  ControlBinop cBinop_i69hPr3i;
  ControlBinop cBinop_vrc9B5Pt;
  ControlVar cVar_NvfhmRCv;
  ControlBinop cBinop_F4mBUkgS;
  SignalVarf sVarf_oOIk4ZLn;
  SignalVarf sVarf_vJXic21d;
  SignalVarf sVarf_x5XffQVK;
  SignalVarf sVarf_SzfaGnTm;
  ControlBinop cBinop_sR5pdsV7;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
